#include <acado_optimal_control.hpp>
#include <include/acado_gnuplot/gnuplot_window.hpp>


int main( ){

    USING_NAMESPACE_ACADO;

    // DEFINE SOME CONSTANTS:
    // -------------------------
    
    const double t_start = 0.0; // start time
    const double t_end   = 1.0; // end time
    const double l       = 1.0; // position of the wall


    // INTRODUCE THE VARIABLES:
    // -------------------------
    
    DifferentialState    x,v ;
    Control              u   ;
    DifferentialEquation f(t_start,t_end);

    // DEFINE A DIFFERENTIAL EQUATION:
    // -------------------------------
    
    f << dot(v) ...; // Write the equations
    f << ...;
    
    // DEFINE AN OPTIMAL CONTROL PROBLEM:
    // ----------------------------------
    const int numberOfIntervals = 40;

    OCP ocp( t_start, t_end, numberOfIntervals );

    ocp.minimizeLagrangeTerm( ); // select and implement the cost function that applies
    ocp.minimizeMayerTerm( );

    ocp.subjectTo( f );  // differential equation as constraint

    ocp.subjectTo( AT_START, x==0.0); // Add start constraints as many as needed

    ocp.subjectTo( AT_END, ... );  // Add end constraints as many needed

    ocp.subjectTo( ... );           // Path constraints

    // PREPARE the window

    GnuplotWindow window;
        window.addSubplot( x, "THE DISTANCE x"      );
        window.addSubplot( v, "THE VELOCITY v"      );
        window.addSubplot( u, "THE CONTROL INPUT u" );

    // DEFINE AN OPTIMIZATION ALGORITHM AND SOLVE THE OCP:
    // ---------------------------------------------------
    OptimizationAlgorithm algorithm(ocp);

    algorithm << window;

//    algorithm.set( HESSIAN_APPROXIMATION, EXACT_HESSIAN );

    algorithm.solve();

    return 0;
}
